<?php

namespace Drupal\refields;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;

trait EntityHelpers {

  /**
   * List all bundles for a given entity type.
   *
   * @param string $entity_type_id
   *
   * @return array
   */
  protected function listBundles(string $entity_type_id): array {
    /** @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info */
    $entity_type_bundle_info = \Drupal::service(EntityTypeBundleInfoInterface::class);

    return array_map(
      fn(array $info) => $info['label'],
      $entity_type_bundle_info->getBundleInfo($entity_type_id)
    );
  }

  /**
   * Check if a field exists on a given entity type and bundle.
   *
   * @param string $field_name
   * @param string $entity_type_id
   * @param string $bundle
   *
   * @return bool
   */
  protected function fieldExists(
    string $field_name,
    string $entity_type_id,
    string $bundle
  ): bool {
    /** @var \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager */
    $entity_field_manager = \Drupal::service(EntityFieldManagerInterface::class);

    $field_definitions = $entity_field_manager->getFieldDefinitions(
      $entity_type_id,
      $bundle
    );

    return isset($field_definitions[$field_name]);
  }

}
