<?php

namespace Drupal\refields\Fields;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\field\FieldStorageConfigInterface;

/**
 * Create fields on bundle creation.
 */
final class CreateFieldsOnBundleCreation extends BaseFieldCreator {

  /**
   * Create fields on bundle creation.
   *
   * @param string $entity_type_id
   * @param string $bundle
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createFields(string $entity_type_id, string $bundle): void {
    $field_storage_definitions = $this->listReusableFields($entity_type_id, $bundle);
    foreach ($field_storage_definitions as $field_name => $definition) {
      $this->performFieldSetup($bundle, $definition);
    }
  }

  /**
   * List reusable fields.
   *
   * @param string $entity_type_id
   * @param string $bundle
   *
   * @return \Drupal\field\FieldStorageConfigInterface[]
   */
  private function listReusableFields(
    string $entity_type_id,
    string $bundle
  ): array {
    return array_filter(
      $this->entityFieldManager->getFieldStorageDefinitions($entity_type_id),
      function(FieldStorageDefinitionInterface $definition) use ($entity_type_id, $bundle): bool {
        if (!$definition instanceof FieldStorageConfigInterface) {
          return FALSE;
        }

        $settings = $definition->getThirdPartySettings('refields');
        if (empty($settings)) {
          return FALSE;
        }

        if (!$settings['refields_enabled']) {
          return FALSE;
        }

        return !in_array(
          $bundle,
          $settings['refields_refields_excluded_bundles'] ?? []
        );
      }
    );
  }

}
