<?php

namespace Drupal\refields\Fields;

use Drupal\Core\Field\FieldStorageDefinitionInterface;

final class CreateFieldsOnFieldConfigUpdate extends BaseFieldCreator {

  /**
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createFields(FieldStorageDefinitionInterface $definition): void {
    $applicable_bundles = $this->listApplicableBundles($definition);
    foreach ($applicable_bundles as $bundle) {
      $field_name = $definition->getName();
      $entity_type_id = $definition->getTargetEntityTypeId();
      if ($this->fieldExists($field_name, $entity_type_id, $bundle)) {
        continue;
      }

      $this->performFieldSetup($bundle, $definition);
    }
  }

  private function listApplicableBundles(FieldStorageDefinitionInterface $definition): array {
    return array_filter(
      array_keys($this->listBundles($definition->getTargetEntityTypeId())),
      function(string $bundle) use ($definition): bool {
        $excluded_bundles = $definition->getThirdPartySetting(
          'refields',
          'refields_excluded_bundles'
          , []
        );

        return !in_array($bundle, $excluded_bundles);
      }
    );
  }

}
